==> emelfm2-0.1.2.tar.gz: emelfm2-0.1.2/src/utils/e2_list.c <==
/* $Id: e2_list.c 762 2005-05-13 04:00:38Z tpgww $
 *
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file e2_list.c 
 * @brief glist utilities
 *
 * glist utility-functions
 *
 **/

#include "emelfm2.h"
#include <string.h>

/**
 * @brief compare function for glists with strings as their data
 * 
 * compare function for glists with strings as their data
 *
 * @param a  first string
 * @param b  second string
 *
 * @return FALSE if the strings are equal, TRUE otherwise
 **/
gboolean e2_list_strcmp (gchar *a, gchar *b)
{
	return !g_str_equal (a, b);
}
/**
 * @brief
 * 
 *
 *
 * @param command
 * @param history
 * @param cur
 * @param max
 * @param doubl
 *
 * @return
 **/
void e2_list_update_history (const gchar *command, GList **history, gint *cur,
	gint max, gboolean doubl)
{
	GList *tmp = doubl ? NULL : g_list_find_custom (*history, command,
		(GCompareFunc) e2_list_strcmp);

	//found the command in history
	if (tmp != NULL)
	{
		*history = g_list_remove_link (*history, tmp);
		*history = g_list_append (*history, g_strdup (command));
		g_free (tmp->data);
		g_list_free_1 (tmp);
	} else
	//didn't find the command in history
	{
		if ((*history != NULL) && (g_str_equal (command,
			g_list_nth_data (*history, g_list_length (*history) - 1))))
			return;
		*history = g_list_append (*history, g_strdup (command));
		if (g_list_length (*history) > max)
		{
			tmp = g_list_nth (*history, 0);
			*history = g_list_remove_link (*history, tmp);
			g_free (tmp->data);
			g_list_free_1 (tmp);
		}
	}
	if (cur != NULL)
		*cur = g_list_length (*history);
}
/**
 * @brief convert a GList of strings to an array of strings
 *
 * convert a GList of strings to an NULL-terminated array of strings
 *
 * @param list GList with gchar pointers as data
 *
 * @return NULL-terminated array of strings from @a list
 **/
gchar **e2_list_to_strv (GList *list)
{
	gint len = g_list_length (list);
	gchar **strv = g_malloc ((len + 1) * sizeof (gpointer));
	gint i;
	GList *l = list;
	for (i = 0; i < len; i++)
	{
		strv[i] = l->data;
		l = g_list_next (l);
	}
	strv[len] = NULL;
	return strv;
}
/**
 * @brief copy a GList of strings
 *
 * This function copies a GList and also copies the data of each element
 * with g_strdup.
 *
 * @param list the GList to copy
 *
 * @return new GList that has to be freed
 **/
GList *e2_list_copy_with_data (GList *list)
{
	GList *copy = NULL;
	GList *tmp;
	for (tmp = g_list_first (list); tmp != NULL; tmp = g_list_next (tmp))
	{
		copy = g_list_append (copy, g_strdup (tmp->data));
	}
	return copy;
}
/**
 * @brief
 * 
 *
 *
 * @param list
 *
 * @return
 **/
void e2_list_free_with_data (GList **list)
{
	if ((list != NULL) && (*list != NULL))
	{
		g_list_foreach (*list, (GFunc) g_free, NULL);
		g_list_free (*list);
		*list = NULL;
	}
}
/**
 * @brief
 * 
 *
 *
 * @param list
 *
 * @return
 **/
void e2_list_free_data_only (GList **list)
{
	if ((list != NULL) && (*list != NULL))
	{
		g_list_foreach (*list, (GFunc) g_free, NULL);
		*list = NULL;
	}
}
/**
 * @brief
 * 
 *
 *
 * @param list
 * @param search_text
 *
 * @return Glist *
 **/
GList *
string_glist_find (GList *list, gchar *search_text)
{
	GList *tmp;
	gchar *curr;

	for (tmp = list; tmp != NULL; tmp = tmp->next)
	{
		curr = tmp->data;
		if (g_str_equal (curr, search_text)) return tmp;
	}
	return NULL;
}

==> emelfm2-0.1.4.tar.gz: emelfm2-0.1.4/src/utils/e2_list.c <==
/* $Id: e2_list.c 1146 2005-09-16 02:30:55Z tpgww $
 *
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file e2_list.c 
 * @brief glist utilities
 *
 * glist utility-functions
 *
 **/

#include "emelfm2.h"
#include <string.h>

/**
 * @brief compare function for glists with strings as their data
 * 
 * compare function for glists with strings as their data
 *
 * @param a  first string
 * @param b  second string
 *
 * @return FALSE if the strings are equal, TRUE otherwise
 **/
gboolean e2_list_strcmp (gchar *a, gchar *b)
{
	return !g_str_equal (a, b);
}
/**
 * @brief
 * 
 *
 *
 * @param command
 * @param history
 * @param cur
 * @param max
 * @param doubl
 *
 * @return
 **/
void e2_list_update_history (const gchar *command, GList **history,
	guint *cur, guint max, gboolean doubl)
{
	GList *tmp = doubl ? NULL : g_list_find_custom (*history, command,
		(GCompareFunc) e2_list_strcmp);

	//found the command in history
	if (tmp != NULL)
	{
		*history = g_list_remove_link (*history, tmp);
		*history = g_list_append (*history, g_strdup (command));
		g_free (tmp->data);
		g_list_free_1 (tmp);
	} else
	//didn't find the command in history
	{
		if ((*history != NULL) && (g_str_equal (command,
			g_list_nth_data (*history, g_list_length (*history) - 1))))
			return;
		*history = g_list_append (*history, g_strdup (command));
		if (g_list_length (*history) > max)
		{
			tmp = g_list_nth (*history, 0);
			*history = g_list_remove_link (*history, tmp);
			g_free (tmp->data);
			g_list_free_1 (tmp);
		}
	}
	if (cur != NULL)
		*cur = g_list_length (*history);
}
/**
 * @brief convert a GList of strings to an array of strings
 *
 * convert a GList of strings to an NULL-terminated array of strings
 *
 * @param list GList with gchar pointers as data
 *
 * @return NULL-terminated array of strings from @a list
 **/
gchar **e2_list_to_strv (GList *list)
{
	gint len = g_list_length (list);
	gchar **strv = g_malloc ((len + 1) * sizeof (gpointer));
	gint i;
	GList *l = list;
	for (i = 0; i < len; i++)
	{
		strv[i] = l->data;
		l = g_list_next (l);
	}
	strv[len] = NULL;
	return strv;
}
/**
 * @brief copy a GList of strings
 *
 * This function copies a GList and also copies the data of each element
 * with g_strdup.
 *
 * @param list the GList to copy
 *
 * @return new GList that has to be freed
 **/
GList *e2_list_copy_with_data (GList *list)
{
	GList *copy = NULL;
	GList *tmp;
	for (tmp = g_list_first (list); tmp != NULL; tmp = g_list_next (tmp))
	{
		copy = g_list_append (copy, g_strdup (tmp->data));
	}
	return copy;
}
/**
 * @brief
 * 
 *
 *
 * @param list
 *
 * @return
 **/
void e2_list_free_with_data (GList **list)
{
	if ((list != NULL) && (*list != NULL))
	{
		g_list_foreach (*list, (GFunc) g_free, NULL);
		g_list_free (*list);
		*list = NULL;
	}
}
/**
 * @brief
 * 
 *
 *
 * @param list
 *
 * @return
 **/
void e2_list_free_data_only (GList **list)
{
	if ((list != NULL) && (*list != NULL))
	{
		g_list_foreach (*list, (GFunc) g_free, NULL);
		*list = NULL;
	}
}
/**
 * @brief
 * 
 *
 *
 * @param list
 * @param search_text
 *
 * @return Glist *
 **/
GList *
string_glist_find (GList *list, gchar *search_text)
{
	GList *tmp;
	gchar *curr;

	for (tmp = list; tmp != NULL; tmp = tmp->next)
	{
		curr = tmp->data;
		if (g_str_equal (curr, search_text)) return tmp;
	}
	return NULL;
}
